diff -Naur asterisk-original/channels/chan_sip.c asterisk-new/channels/chan_sip.c
--- asterisk-original/channels/chan_sip.c	2006-03-23 18:12:14.000000000 -0500
+++ asterisk-new/channels/chan_sip.c	2006-03-23 18:16:43.000000000 -0500
@@ -401,6 +401,7 @@
 #define DEFAULT_PEDANTIC	FALSE
 #define DEFAULT_AUTOCREATEPEER	FALSE
 #define DEFAULT_QUALIFY		FALSE
+#define DEFAULT_REGEXTENONQUALIFY FALSE
 #define DEFAULT_T1MIN		100		/*!< 100 MS for minimal roundtrip time */
 #define DEFAULT_MAX_CALL_BITRATE (384)		/*!< Max bitrate for video */
 #ifndef DEFAULT_USERAGENT
@@ -451,6 +452,7 @@
 static char default_parkinglot[AST_MAX_EXTENSION];	/*!< Default parking lot */
 static int global_callevents;		/*!< Whether we send manager events or not */
 static int global_t1min;		/*!< T1 roundtrip time minimum */
+static int global_regextenonqualify;	/*!< Whether to add/remove regexten when qualifying peers */
 
 /*! \brief Codecs that we support by default: */
 static int global_capability = AST_FORMAT_ULAW | AST_FORMAT_ALAW | AST_FORMAT_GSM | AST_FORMAT_H263;
@@ -9099,6 +9101,7 @@
 	ast_cli(fd, "  User Agent:             %s\n", global_useragent);
 	ast_cli(fd, "  MWI checking interval:  %d secs\n", global_mwitime);
 	ast_cli(fd, "  Reg. context:           %s\n", ast_strlen_zero(global_regcontext) ? "(not set)" : global_regcontext);
+	ast_cli(fd, "  Regexten on Qualify:    %s\n", global_regextenonqualify ? "Yes" : "No");
 	ast_cli(fd, "  Caller ID:              %s\n", default_callerid);
 	ast_cli(fd, "  From: Domain:           %s\n", default_fromdomain);
 	ast_cli(fd, "  Record SIP history:     %s\n", recordhistory ? "On" : "Off");
@@ -10684,6 +10687,8 @@
 		if ((peer->lastms < 0)  || (peer->lastms > peer->maxms)) {
 			if (pingtime <= peer->maxms) {
 				ast_log(LOG_NOTICE, "Peer '%s' is now REACHABLE! (%dms / %dms)\n", peer->name, pingtime, peer->maxms);
+				if (global_regextenonqualify)
+					register_peer_exten(peer, 1);
 				statechanged = 1;
 				newstate = 1;
 			}
@@ -12479,6 +12484,8 @@
 	if (peer->lastms > -1) {
 		ast_log(LOG_NOTICE, "Peer '%s' is now UNREACHABLE!  Last qualify: %d\n", peer->name, peer->lastms);
 		manager_event(EVENT_FLAG_SYSTEM, "PeerStatus", "Peer: SIP/%s\r\nPeerStatus: Unreachable\r\nTime: %d\r\n", peer->name, -1);
+		if (global_regextenonqualify)
+			register_peer_exten(peer, 0);
 	}
 	if (peer->call)
 		sip_destroy(peer->call);
@@ -13481,6 +13488,7 @@
 	/* Reset channel settings to default before re-configuring */
 	allow_external_domains = DEFAULT_ALLOW_EXT_DOM;				/* Allow external invites */
 	global_regcontext[0] = '\0';
+	global_regextenonqualify = DEFAULT_REGEXTENONQUALIFY;
 	expiry = DEFAULT_EXPIRY;
 	global_notifyringing = DEFAULT_NOTIFYRINGING;
 	global_allowsubscribe = TRUE;
@@ -13615,6 +13623,8 @@
 			/* Create context if it doesn't exist already */
 			if (!ast_context_find(global_regcontext))
 				ast_context_create(NULL, global_regcontext, "SIP");
+		} else if (!strcasecmp(v->name, "regextenonqualify")) {
+			global_regextenonqualify = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "callerid")) {
 			ast_copy_string(default_callerid, v->value, sizeof(default_callerid));
 		} else if (!strcasecmp(v->name, "fromdomain")) {
